@@ -24,6 +24,7 @@ import zipfile
 import pkg_resources
 
 from sushy import exceptions
+from sushy.resources import mappings as res_maps
 from sushy.resources import oem
 from sushy import utils
 
@@ -213,6 +214,9 @@ class ListField(Field):
 
         return instances
 
+    def __getitem__(self, key):
+        return getattr(self, key)
+
 
 class DictionaryField(Field):
     """Base class for fields consisting of dictionary of several sub-fields."""
@@ -246,6 +250,9 @@ class DictionaryField(Field):
 
         return instances
 
+    def __getitem__(self, key):
+        return getattr(self, key)
+
 
 class MappedField(Field):
     """Field taking real value from a mapping."""
@@ -315,7 +322,61 @@ class MappedListField(Field):
         return instances
 
 
-class AbstractJsonReader(object, metaclass=abc.ABCMeta):
+class MessageListField(ListField):
+    """List of messages with details of settings update status"""
+
+    message_id = Field('MessageId', required=True)
+    """The key for this message which can be used
+    to look up the message in a message registry
+    """
+
+    message = Field('Message')
+    """Human readable message, if provided"""
+
+    severity = MappedField('Severity',
+                           res_maps.SEVERITY_VALUE_MAP)
+    """Severity of the error"""
+
+    resolution = Field('Resolution')
+    """Used to provide suggestions on how to resolve
+    the situation that caused the error
+    """
+
+    _related_properties = Field('RelatedProperties')
+    """List of properties described by the message"""
+
+    message_args = Field('MessageArgs')
+    """List of message substitution arguments for the message
+    referenced by `message_id` from the message registry
+    """
+
+
+class FieldData(object):
+    """Contains data to be used when constructing Fields"""
+
+    def __init__(self, status_code, headers, json_doc):
+        """Initializes the FieldData instance"""
+        self._status_code = status_code
+        self._headers = headers
+        self._json_doc = json_doc
+
+    @property
+    def status_code(self):
+        """The status code"""
+        return self._status_code
+
+    @property
+    def headers(self):
+        """The headers"""
+        return self._headers
+
+    @property
+    def json_doc(self):
+        """The parsed JSON body"""
+        return self._json_doc
+
+
+class AbstractDataReader(object, metaclass=abc.ABCMeta):
 
     def set_connection(self, connector, path):
         """Sets mandatory connection parameters
@@ -327,28 +388,33 @@ class AbstractJsonReader(object, metaclass=abc.ABCMeta):
         self._path = path
 
     @abc.abstractmethod
-    def get_json(self):
+    def get_data(self):
         """Based on data source get data and parse to JSON"""
 
 
-class JsonDataReader(AbstractJsonReader):
+class JsonDataReader(AbstractDataReader):
     """Gets the data from HTTP response given by path"""
 
-    def get_json(self):
+    def get_data(self):
         """Gets JSON file from URI directly"""
         data = self._conn.get(path=self._path)
-        return data.json() if data.content else {}
+
+        json_data = data.json() if data.content else {}
+
+        return FieldData(data.status_code, data.headers, json_data)
 
 
-class JsonPublicFileReader(AbstractJsonReader):
+class JsonPublicFileReader(AbstractDataReader):
     """Loads the data from the Internet"""
 
-    def get_json(self):
+    def get_data(self):
         """Get JSON file from full URI"""
-        return self._conn.get(self._path).json()
+        data = self._conn.get(self._path)
+
+        return FieldData(data.status_code, data.headers, data.json())
 
 
-class JsonArchiveReader(AbstractJsonReader):
+class JsonArchiveReader(AbstractDataReader):
     """Gets the data from JSON file in archive"""
 
     def __init__(self, archive_file):
@@ -358,15 +424,16 @@ class JsonArchiveReader(AbstractJsonReader):
         """
         self._archive_file = archive_file
 
-    def get_json(self):
+    def get_data(self):
         """Gets JSON file from archive. Currently supporting ZIP only"""
 
         data = self._conn.get(path=self._path)
         if data.headers.get('content-type') == 'application/zip':
             try:
                 archive = zipfile.ZipFile(io.BytesIO(data.content))
-                return json.loads(archive.read(self._archive_file)
-                                  .decode(encoding='utf-8'))
+                json_data = json.loads(archive.read(self._archive_file)
+                                       .decode(encoding='utf-8'))
+                return FieldData(data.status_code, data.headers, json_data)
             except (zipfile.BadZipfile, ValueError) as e:
                 raise exceptions.ArchiveParsingError(
                     path=self._path, error=e)
@@ -374,8 +441,10 @@ class JsonArchiveReader(AbstractJsonReader):
             LOG.error('Support for %(type)s not implemented',
                       {'type': data.headers['content-type']})
 
+            return FieldData(data.status_code, data.headers, None)
 
-class JsonPackagedFileReader(AbstractJsonReader):
+
+class JsonPackagedFileReader(AbstractDataReader):
     """Gets the data from packaged file given by path"""
 
     def __init__(self, resource_package_name):
@@ -385,12 +454,28 @@ class JsonPackagedFileReader(AbstractJsonReader):
         """
         self._resource_package_name = resource_package_name
 
-    def get_json(self):
+    def get_data(self):
         """Gets JSON file from packaged file denoted by path"""
 
         with pkg_resources.resource_stream(self._resource_package_name,
                                            self._path) as resource:
-            return json.loads(resource.read().decode(encoding='utf-8'))
+            json_data = json.loads(resource.read().decode(encoding='utf-8'))
+            return FieldData(None, None, json_data)
+
+
+def get_reader(connector, path, reader=None):
+    """Create and configure the reader.
+
+    :param connector: A Connector instance
+    :param path: sub-URI path to the resource.
+    :param reader: Reader to use to fetch JSON data.
+    :returns: the reader
+    """
+    if reader is None:
+        reader = JsonDataReader()
+    reader.set_connection(connector, path)
+
+    return reader
 
 
 class ResourceBase(object, metaclass=abc.ABCMeta):
@@ -406,7 +491,8 @@ class ResourceBase(object, metaclass=abc.ABCMeta):
                  path='',
                  redfish_version=None,
                  registries=None,
-                 reader=None):
+                 reader=None,
+                 json_doc=None):
         """A class representing the base of any Redfish resource
 
         Invokes the ``refresh()`` method of resource for the first
@@ -418,6 +504,7 @@ class ResourceBase(object, metaclass=abc.ABCMeta):
         :param registries: Dict of Redfish Message Registry objects to be
             used in any resource that needs registries to parse messages
         :param reader: Reader to use to fetch JSON data.
+        :param json_doc: parsed JSON document in form of Python types.
         """
         self._conn = connector
         self._path = path
@@ -429,12 +516,34 @@ class ResourceBase(object, metaclass=abc.ABCMeta):
         # attribute values are fetched.
         self._is_stale = True
 
-        if reader is None:
-            reader = JsonDataReader()
-        reader.set_connection(connector, path)
-        self._reader = reader
+        self._reader = get_reader(connector, path, reader)
 
-        self.refresh()
+        self.refresh(json_doc=json_doc)
+
+    def _get_value(self, val):
+        """Iterate through the input to get values for all attributes
+
+        :param val: Either a value or a resource
+        :returns: Attribute value, which may be a dictionary
+        """
+        if isinstance(val, dict):
+            subfields = {}
+            for key, s_val in val.items():
+                subfields[key] = self._get_value(s_val)
+            return subfields
+
+        elif isinstance(val, list):
+            return [self._get_value(val[i]) for i in range(len(val))]
+
+        elif (isinstance(val, DictionaryField)
+              or isinstance(val, CompositeField)
+              or isinstance(val, ListField)):
+            subfields = {}
+            for attr, field in val._subfields.items():
+                subfields[attr] = self._get_value(val.__getitem__(attr))
+            return subfields
+
+        return val
 
     def _parse_attributes(self, json_doc):
         """Parse the attributes of a resource.
@@ -442,12 +551,19 @@ class ResourceBase(object, metaclass=abc.ABCMeta):
         Parsed JSON fields are set to `self` as declared in the class.
 
         :param json_doc: parsed JSON document in form of Python types
+        :returns: dictionary of attribute/values after parsing
         """
+        settings = {}
         for attr, field in _collect_fields(self):
             # Hide the Field object behind the real value
             setattr(self, attr, field._load(json_doc, self))
 
-    def refresh(self, force=True):
+            # Get the attribute/value pairs that have been parsed
+            settings[attr] = self._get_value(getattr(self, attr))
+
+        return settings
+
+    def refresh(self, force=True, json_doc=None):
         """Refresh the resource
 
         Freshly retrieves/fetches the resource attributes and invokes
@@ -460,6 +576,7 @@ class ResourceBase(object, metaclass=abc.ABCMeta):
         :param force: if set to False, will only refresh if the resource is
             marked as stale, otherwise neither it nor its subresources will
             be refreshed.
+        :param json_doc: parsed JSON document in form of Python types.
         :raises: ResourceNotFoundError
         :raises: ConnectionError
         :raises: HTTPError
@@ -469,12 +586,15 @@ class ResourceBase(object, metaclass=abc.ABCMeta):
         if not self._is_stale and not force:
             return
 
-        self._json = self._reader.get_json()
+        if json_doc:
+            self._json = json_doc
+        else:
+            self._json = self._reader.get_data().json_doc
 
+        attributes = self._parse_attributes(self._json)
         LOG.debug('Received representation of %(type)s %(path)s: %(json)s',
                   {'type': self.__class__.__name__,
-                   'path': self._path, 'json': self._json})
-        self._parse_attributes(self._json)
+                   'path': self._path, 'json': attributes})
         self._do_refresh(force)
 
         # Mark it fresh
