@@ -285,6 +285,7 @@ class ConnectorOpTestCase(base.TestCase):
         response = self.conn._op('POST', path='fake/path', data=self.data,
                                  headers=self.headers)
         self.auth.refresh_session.assert_called_with()
+        self.auth.can_refresh_session.assert_called_with()
         self.assertEqual(response.json, second_response.json)
 
     def test_connection_error(self):
@@ -316,6 +317,21 @@ class ConnectorOpTestCase(base.TestCase):
         self.assertIsNotNone(exc.body)
         self.assertIn('body submitted was malformed JSON', exc.detail)
 
+    def test_known_http_error_nonlist_ext_info(self):
+        self.request.return_value.status_code =\
+            http_client.UNSUPPORTED_MEDIA_TYPE
+        with open('sushy/tests/unit/json_samples/'
+                  'error_single_ext_info.json') as f:
+            self.request.return_value.json.return_value = json.load(f)
+
+        with self.assertRaisesRegex(exceptions.HTTPError,
+                                    'See Resolution for information') as cm:
+            self.conn._op('POST', 'http://foo.bar')
+        exc = cm.exception
+        self.assertEqual(http_client.UNSUPPORTED_MEDIA_TYPE, exc.status_code)
+        self.assertIsNotNone(exc.body)
+        self.assertIn('See Resolution for information', exc.detail)
+
     def test_not_found_error(self):
         self.request.return_value.status_code = http_client.NOT_FOUND
         self.request.return_value.json.side_effect = ValueError('no json')
@@ -355,3 +371,23 @@ class ConnectorOpTestCase(base.TestCase):
         with self.assertRaisesRegex(exceptions.ConnectionError,
                                     'status 202, but no Location header'):
             self.conn._op('POST', 'http://foo.bar', blocking=True)
+
+    @mock.patch('sushy.connector.time.sleep', autospec=True)
+    def test_blocking_task_fails(self, mock_sleep):
+        response1 = mock.MagicMock(spec=requests.Response)
+        response1.status_code = http_client.ACCEPTED
+        response1.headers = {
+            'retry-after': 5,
+            'location': '/redfish/v1/taskmon/1'
+        }
+        response2 = mock.MagicMock(spec=requests.Response)
+        response2.status_code = http_client.BAD_REQUEST
+        message = 'Unable to create Volume with given parameters'
+        response2.json.return_value = {
+            'error': {
+                'message': message
+            }
+        }
+        self.request.side_effect = [response1, response1, response2]
+        with self.assertRaisesRegex(exceptions.BadRequestError, message):
+            self.conn._op('POST', 'http://foo.bar', blocking=True)
